"""
Classifies: CHEBI:17517 phosphatidylglycerol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_phosphatidylglycerol(smiles: str):
    """
    Determines if a molecule is a phosphatidylglycerol.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a phosphatidylglycerol, False otherwise
        str: Reason for classification
    """
    
    try:
        mol = Chem.MolFromSmiles(smiles)
        if mol is None:
            return False, "Invalid SMILES string"

        # Check for presence of phosphorus
        atoms = [atom.GetSymbol() for atom in mol.GetAtoms()]
        if 'P' not in atoms:
            return False, "No phosphorus atom found"
            
        # Check for presence of phosphate group
        phosphate_pattern = Chem.MolFromSmarts('[P](=O)([O,OH])[O,OH]')
        if not mol.HasSubstructMatch(phosphate_pattern):
            return False, "No phosphate group found"
            
        # Check for glycerol backbone with phosphate
        glycerol_phosphate = Chem.MolFromSmarts('[#6]-[#6]-[#6]-O-P(=O)([O,OH])-O-[#6]-[#6]-[#6]')
        if not mol.HasSubstructMatch(glycerol_phosphate):
            return False, "No glycerol-phosphate backbone found"

        # Check for ester linkages
        ester_pattern = Chem.MolFromSmarts('C(=O)-O')
        ester_count = len(mol.GetSubstructMatches(ester_pattern))
        if ester_count < 1:
            return False, "Missing ester linkages"

        # Look for terminal glycerol moiety
        terminal_glycerol = Chem.MolFromSmarts('O-C-C(O)-CO')
        if not mol.HasSubstructMatch(terminal_glycerol):
            # Try alternative pattern for masked terminal groups
            alt_terminal = Chem.MolFromSmarts('O-C-C-C-O')
            if not mol.HasSubstructMatch(alt_terminal):
                return False, "Missing terminal glycerol group"

        # Special check for generic/masked structures
        if '*' in smiles:
            return True, "Valid phosphatidylglycerol structure with masked groups"

        # Check for fatty acid chains or their masked representations
        carbon_chain = Chem.MolFromSmarts('CCCC')  # Shorter chain requirement
        if not mol.HasSubstructMatch(carbon_chain) and '*' not in smiles:
            return False, "No acyl chains found"

        return True, "Valid phosphatidylglycerol structure identified"

    except Exception as e:
        return None, f"Error analyzing molecule: {str(e)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17517',
                          'name': 'phosphatidylglycerol',
                          'definition': 'A glycerophosphoglycerol that is '
                                        'glycerol in which the hydrogen of one '
                                        'of the primary hydroxy groups has '
                                        'been replaced by a phosphatidyl '
                                        'group.',
                          'parents': ['CHEBI:24360']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.4126984126984127 is too low.\n'
               'True positives: '
               "[('C([C@@](COC(CCCCCCCCC/C=C\\\\CCCCCC)=O)(OC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCC(C)C)COC(=O)CCCCCCCC(O)/C=C/C=C/C/C=C/CC)(OC[C@@H](O)COP(O)(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(OC[C@@H](O)COP(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCC)(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCCCCCCCCCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCCCC)=O)(OC(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@@H](O)COP(O)(O)=O)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@@H](O)CO)OC(=O)CCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)OCC(COP(O)(=O)OCC(O)CO)OC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@@H](O)CO)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCC)COC(=O)CCCCCCCCCC)(OC[C@@H](O)COP(OC[C@H](OC(=O)CCCCCCC)COC(=O)CCCCCCC)(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCCC)(OC[C@H](O)COP(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCCCCCC)(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCCCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCC)COC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C(OP(=O)(OCC(CO)O)O)[C@@H](COC(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)OC(C/C=C/CCCCCCCCCCCC)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCC(=O)OCC(COP(O)(=O)OCC(O)CO)OC(=O)CCCCCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@@H](O)COP(O)(=O)OCC(O)COP(O)(=O)OC[C@H](O)COC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCC(=O)/C=C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCC(CC)C)(OC[C@@H](O)COP(O)(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCCCC/C=C\\\\CCCCCC)=O)(OC(CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@@H](O)CO)OC(=O)CCCCCCCCCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCC(O)/C=C/C=O)COC(=O)CCCCCCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCC)COC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP(=O)(O)OC[C@@](CO)([H])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCC(C)C)COC(=O)CCCC(=O)/C=C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('O(P(OCC(COP(OC[C@@H](COC(CCCCCCCC=CCCCCCC)=O)OC(CCCCCCCC=CCCCCCC)=O)(O)=O)O)(O)=O)C[C@@H](COC(CCCCCCCC=CCCCCCC)=O)OC(CCCCCCCC=CCCCCCC)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCC(=O)O[C@H](COC(=O)CCCCCCC\\\\C=C/CCCCCCCC)COP(O)(=O)OC[C@@H](O)CO', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC(C)C)COC(=O)CCCCCCCCC(CC)C)(OC[C@@H](O)COP(O)(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified')]\n"
               'False positives: '
               "[('P(OC1C(O)C(O)C(O)[C@@H](O)C1O)(OC[C@H](OC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCCCCC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC1C(O)C(O)C(O)[C@@H](O)C1O)(OC[C@H](OC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)CO/C=C\\\\CCCCCCCCCCCCCCCCCC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C(CCCCCCC/C=C\\\\CCCCCCCC)(=O)O[C@@H](COP(=O)(OCCN)O)CO/C=C\\\\CCCCCCCCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCC/C=C\\\\CCCCCC)=O)(OC(CCCCCCCCCCCCCCCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC1C(O)C(O)C(O)[C@@H](O)C1O)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCC)COC(=O)CCCCCCCCCCCCCCCC)([O-])=O.[NH4+]', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C(C[N+](C)(C)C)OP(=O)([O-])OC[C@H](OC(=O)[H])COCCCCCCCCCCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('O(C[C@@H](COP([O-])(=O)OC[C@H](COC(CCCCCCCCCCCCC)=O)O)O)C(CCCCCCCCCCCCC)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCC/C=C\\\\C[C@H]1[C@@H](O)CC(O[C@@H]1/C=C/[C@@H](O)CCCCC)O)COC(=O)CCCCCCCCCCCCCCCC(C)C)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@](COC(=O)CCCCCCCCCCCCCCCCCCCC)(OC(=O)CCCCCCCCCCCCCCC)([H])COP(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC1C(O)C(O)C(O)[C@@H](O)C1O)(OC[C@H](OC(=O)CCCCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCC\\\\C=C\\\\C[C@@H]1[C@H]([C@H](O)C[C@@H]1O)/C=C/[C@@H](O)CCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@H]1([C@H]([C@@H]([C@H]([C@@H]([C@@H]1O)OP(OC[C@H](COC(CCCCCCC)=O)OC(CCCCCCC)=O)(=O)O)O)OP(=O)(O)O)OP(O)(=O)O)O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCCCCC)COC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC1C(O)C(O)C(O)[C@@H](O)C1O)(OC[C@H](OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@](COC(=O)CCCCCCC/C=C\\\\CCCC)(OC(=O)CCCCCCCCCCCCC/C=C\\\\CCCCCCCC)([H])COP(OCC[N+](C)(C)C)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](O)COCC(OC)CCCCCCCCCCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@@H](COC(=O)CCCCCCCCCCCCCCC)(COP(OCC[N+](C)(C)C)(=O)[O-])O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O)O)O)O*)OP(OC[C@H](CO*)OC(*)=O)(=O)[O-])O[C@@H]2[C@H]([NH3+])[C@@H](O)[C@@H]([C@H](O2)CO)O[C@@H]3[C@H]([C@H]([C@@H]([C@H](O3)CO[C@@H]4[C@H]([C@H]([C@@H]([C@H](O4)CO*)O)O)O[C@@H]5[C@H]([C@H]([C@@H]([C@H](O5)COP([O-])(=O)OCCNC([C@@H](N*)CO)=O)O)O)O*)O*)O*)O*', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(=O)(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COC(=O)CCCCCCCC[C@@H](CCCCCCCC)C)(OCCN)O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCCC)(OC[C@H]([NH3+])C([O-])=O)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCC(=O)OC[C@@H](O)COP([O-])(=O)OC[C@@H](O)CO', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCCCCCC/C=C\\\\CCCCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O)O)O)OC(=O)*)OP(OC[C@H](CO*)OC(=O)*)(=O)[O-])O[C@@H]2[C@H]([NH3+])[C@@H](O)[C@@H]([C@H](O2)CO)O[C@@H]3[C@H]([C@H]([C@@H]([C@H](O3)CO[C@@H]4[C@H]([C@H]([C@@H]([C@H](O4)COP([O-])(=O)OCC[NH3+])O)O)O[C@@H]5[C@H]([C@H]([C@@H]([C@H](O5)COP([O-])(=O)OCC[NH3+])O)O)O)O)O)OP([O-])(=O)OCC[NH3+]', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[S+](CCOP(OC[C@H](OC(=O)CCCCCCCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCCC)([O-])=O)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C(C(COC(CCCCCCCCC/C=C\\\\CCCCCCCC)=O)O/C=C\\\\CCCCCCCC/C=C\\\\CCCCCC)OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC(OC(=O)CCCCCCCCCCCCC)COC(=O)CCCCCCCCCCC/C=C\\\\CCCCCCCC)(OCCN(C)C)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@H](N)C(O)=O)OC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@](COC(=O)CCCCCCC/C=C\\\\CCCC)(O)([H])COP(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCCCCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OCCN)OC(=O)CCCCCCCCCCCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCCCC)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCC)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCC)COCCCCCCCCCCCCCCCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC(OC(=O)CCCCCCCCC/C=C\\\\CCCCCC)COC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(OCCN(C)C)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[H]C(=O)CCCC(=O)O[C@H](COC(=O)CCCCC)COP(O)(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC1C(O)C(O)C(O)[C@@H](O)C1O)(OC[C@H](O)COC(=O)CCCCCCC/C=C\\\\CCCCC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CCCCCCCCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)O[C@H](COC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC)COP([O-])(=O)OCC(O)COP([O-])(=O)OC[C@@H](COC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC)OC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@](COC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(OC(=O)CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)([H])COP(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCOC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(C)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC(OC(=O)CCCCCCCCCCCCCCC)COC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)(OCCNC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCCC(=O)O[C@H](CO)COP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCC)CO/C=C\\\\CCCCCCCCCCCCCCCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCCCCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('O([C@H](COC(=O)CCCCCCCCCCCCCCC)COP(OCC[N+](C)(C)C)(=O)[O-])C(=O)CCCCCCCCO', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@](COC(=O)CCCCCCC/C=C\\\\CCCCCC)(O)([H])COP(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@@H](O)COP([O-])(=O)OCC(O)COP([O-])(=O)OC[C@H](O)COC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCCCC/C=C\\\\CCCCCCCC)=O)(OC(CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\CCCCCCCC)COC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)O[C@H](COC([*])=O)COP([O-])(=O)OC[C@H]([NH3+])C([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC(OC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(OCCN(C)C)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](O)COC(=O)CCCCCCC/C=C\\\\CCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@@H](OC(CCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(COC(=O)CCCCCCCCCCCCCCC)COP(O[C@H]1[C@@H]([C@H]([C@@H]([C@H]([C@H]1O)O)O)O)O)(=O)O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@@H](CO/C=C\\\\CCCCCCCCCCCCCCCC)OC(CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(=O)(OCCN)O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCC\\\\C=C\\\\CCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CCCCCCC\\\\C=C\\\\CCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@H](COP([O-])(=O)OC[C@@H](O)CO)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(O[C@@H]1C(O)[C@H](OP(O)(O)=O)C(O)C(O)C1O)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCCCCCCCC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(=O)(O)(OCCN)OC[C@@H](CO)OC(CCC/C=C\\\\C[C@H]1[C@@H]2C[C@H]([C@@H]1/C=C/[C@H](CCCCC)OO)OO2)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('OC1C(O)C(O)C(OP([O-])(=O)OCC(COC=C[*])OC([*])=O)C(O)C1O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCC[C@@H]1[C@H]([C@H](O)C[C@@H]1O)/C=C/[C@@H](O)CCCCC)COC(=O)CCCCCCCCCCCCCCCCCCCCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCC)COC(=O)CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCC/C=C\\\\CCCCCCCCCC)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCC[C@@H]1[C@H]([C@H](O)C[C@@H]1O)/C=C/[C@@H](O)CCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OC[C@H](N)C(O)=O)OC(=O)CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[H]/C(/CC)=C(/[H])\\\\C/C(/[H])=C(\\\\[H])/C/C(/[H])=C(\\\\[H])/C/C(/[H])=C(\\\\[H])/C/C(/[H])=C(\\\\[H])/CCCCCC(=O)OC[C@]([H])(COP(O)(=O)OCCN)O/C(/[H])=C(\\\\[H])/CCCCCC/C(/[H])=C(\\\\[H])/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@](COC(=O)CCCCCCCCCCCCCCC)(O)([H])COP(OCCN*)(=O)[O-]', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@@H]1(C(C(C([C@H](C1O)O)O)O)O)OP(OC[C@@](COC(CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)=O)(OC(CCCCCCCCC/C=C\\\\CCCCCC)=O)[H])(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCC=1OC(=C(C1C)C)CCCCC)COC(=O)CCCCC(=O)C[C@@H]2[C@H]([C@H](O)C[C@@H]2O)/C=C/[C@@H](O)CCCCC)(OCC[N+](C)(C)C)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\CCCCCCCCCC)COC(=O)CCCCCCCCCCCCCC)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('O(P(=O)(OCC[N+](C)(C)C)[O-])C[C@H](O)COC(CCCCCCCC(O)=O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O)(OC(CCCCCCCCCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('O(P(OC[C@H]1O[C@@H](N2C=3N=CN=C(N)C3N=C2)[C@@H]([C@@H]1O)O)(=O)O)C(CCCCCCCCCCCCCCCCCCC4=CC=C(C=C4)O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCCCCC(=O)NCCOP([O-])(=O)OC[C@H](O)COC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC(OC(=O)CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)(OCCNC)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OCC[N+](C)(C)C)(OC[C@H](OC(=O)CCCCCCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@@H](CO)OC(CCCCCCC/C=C\\\\CC(CCCCCC)O)=O)(=O)(OCC[N+](C)(C)C)[O-]', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCCC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[H][C@@]1(O[C@H](O[C@@H]2[C@H](O)[C@@H](O[C@@H]3[C@@H](C[C@@](O)(O[C@]3([H])[C@H](O)CO)C(O)=O)O[C@@]3(C[C@@H](O)[C@@H](O)[C@]([H])(O3)[C@H](O)CO)C(O)=O)O[C@]([H])([C@@H](O)CO)[C@H]2O[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)[C@@H](O[C@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2NC(C)=O)[C@@H](OP(O)(=O)OCCN)[C@@H]1O)[C@@H](O)CO', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](O)COC(=O)CCCCCCCCC/C=C\\\\C/C=C\\\\CCCCC)(OC[C@@H](O)CO)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[H][C@@](COC(=O)CCCCCCCCCCCCCCC)(COP(O)(=O)OCCN)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('N(CCOP(OC[C@@H](COC(CCCCCCCCCCCCCCCC)=O)OC(CCCCCCCCCCCCCCCC)=O)(=O)O)C(=O)CCCCCCC/C=C\\\\CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)NCCOP([O-])(=O)OC[C@H](O)COC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C([C@@](COC(CCCCCCCCC/C=C\\\\CCCCCC)=O)(OC(CCCCCCC/C=C\\\\CCCCCC)=O)[H])OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCCCCCC)COC(=O)CCCCCCCCCCCCCCCCC)(OCCN)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[H][C@](COP(OC[C@@](COC(CCCCCCCCCCCCCCCCC)=O)(OC(CCCCCCCCCCCCCCC)=O)[H])(=O)O)(C(O)=O)N', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)O[C@H](COC([*])=O)COP(O)(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('[C@](COC(=O)CCCCCCC/C=C\\\\CCCC)(O/C=C\\\\CCCCCCCCCCCCCC)([H])COP(OCC[N+](C)(C)C)([O-])=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCCCC)COC(=O)CCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('C(C(COC(CCCCCCCCCCCCC)=O)O/C=C\\\\CCCCCCCCCCCCCCCC)OP([O-])(=O)OCC[N+](C)(C)C', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('O=P([O-])(O[C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O)O)O)O)O)OC[C@@H](COC(CCCCCCC/C=C\\\\CCCCCCCC)=O)OC(CCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('P(OC[C@H](OC(=O)CCCCCCCCC/C=C\\\\CCCCCCCCCC)COC(=O)CCCCCCCCCCCCC)(OC[C@H](N)C(O)=O)(O)=O', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCCCCCCCCCCCCC(=O)OC[C@H](COP(O)(=O)OCCN)OC(=O)CCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC', "
               "'Valid phosphatidylglycerol structure identified'), "
               "('CCCCCC\\\\C=C/CCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)OCC[N+](C)(C)C)OC(=O)CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CC', "
               "'Valid phosphatidylglycerol structure identified')]\n"
               "False negatives: [('C(=O)(OCC(COP(=O)(OCC(CO)O)O)OC(*)=O)*', "
               "'No long carbon chains found'), "
               "('C(=O)(OC[C@H](COP(=O)(OCC(CO)O)O)OC(*)=O)*', 'No long carbon "
               "chains found'), ('C(=O)(OCC(COP(=O)(OCC(CO)O)O)OC(*)=O)*', 'No "
               "long carbon chains found'), "
               "('O(P(=O)(OCC(COP(=O)(O)O)O)O)CC(OC(*)=O)COC(*)=O', 'No long "
               "carbon chains found'), "
               "('C(=O)(OCC(COP(=O)(OCC(CO)O)O)OC(*)=O)*', 'No long carbon "
               "chains found'), "
               "('*C(O[C@@H](COP(OCC(COP(OC[C@H](OC(*)=O)COC(*)=O)(O)=O)O)(O)=O)COC(*)=O)=O', "
               "'No long carbon chains found'), "
               "('*C(O[C@@H](COP(OCC(COP(OC[C@H](OC(*)=O)COC(*)=O)(O)=O)O)(O)=O)COC(*)=O)=O', "
               "'No long carbon chains found'), "
               "('OC(COP(O)(=O)OC[C@@H](COC([*])=O)OC([*])=O)COP(O)(=O)OC[C@@H](COC([*])=O)OC([*])=O', "
               "'No long carbon chains found'), "
               "('C(=O)(OCC(COP(=O)(OCC(CO)O)O)OC(*)=O)*', 'No long carbon "
               "chains found'), ('C(=O)(OCC(COP(=O)(OCC(CO)O)O)OC(*)=O)*', 'No "
               "long carbon chains found'), "
               "('C(=O)(OCC(COP(=O)(OCC(CO)O)O)OC(*)=O)*', 'No long carbon "
               "chains found')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 50,
    'num_false_positives': 100,
    'num_true_negatives': 8981,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.3333333333333333,
    'recall': 1.0,
    'f1': 0.5,
    'accuracy': 0.989048297010185}


"""
Classifies: CHEBI:24402 glycosphingolipid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_glycosphingolipid(smiles: str):
    """
    Determines if a molecule is a glycosphingolipid - a glycolipid that is a carbohydrate-containing 
    derivative of a sphingoid or ceramide, with the carbohydrate residue attached by a glycosidic linkage 
    to O-1 of the sphingoid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycosphingolipid, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for presence of sphingoid/ceramide core
    # Look for characteristic NH-C(=O) amide group and long alkyl chains
    has_amide = False
    has_long_chain = False
    
    # Find amide group
    amide_pattern = Chem.MolFromSmarts('[NH]-C(=O)')
    if mol.HasSubstructMatch(amide_pattern):
        has_amide = True
        
    # Check for long carbon chains by counting connected carbons
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            visited = set()
            chain_length = count_carbon_chain(atom, visited)
            if chain_length >= 10:
                has_long_chain = True
                break
                
    if not (has_amide and has_long_chain):
        return False, "Missing sphingoid/ceramide core structure"

    # Check for carbohydrate residue
    # Look for multiple OH groups and cyclic structures containing O
    rings = mol.GetRingInfo()
    has_sugar = False
    
    for ring in rings.AtomRings():
        if len(ring) >= 5:  # Sugar rings are typically 5 or 6-membered
            ring_atoms = [mol.GetAtomWithIdx(i) for i in ring]
            o_count = sum(1 for a in ring_atoms if a.GetSymbol() == 'O')
            oh_count = sum(1 for a in ring_atoms if a.GetSymbol() == 'C' 
                         and any(n.GetSymbol() == 'O' and n.GetTotalNumHs() > 0 
                               for n in a.GetNeighbors()))
            if o_count >= 1 and oh_count >= 2:
                has_sugar = True
                break
                
    if not has_sugar:
        return False, "Missing carbohydrate residue"

    # Check for glycosidic linkage
    # Look for C-O-C connection between sugar and sphingoid
    glycosidic_pattern = Chem.MolFromSmarts('[C]-O-[C]')
    if not mol.HasSubstructMatch(glycosidic_pattern):
        return False, "Missing glycosidic linkage"

    return True, "Contains sphingoid core, carbohydrate residue, and glycosidic linkage"

def count_carbon_chain(atom, visited):
    """Helper function to count length of connected carbon chains"""
    if atom.GetIdx() in visited or atom.GetSymbol() != 'C':
        return 0
    
    visited.add(atom.GetIdx())
    max_length = 1
    
    for neighbor in atom.GetNeighbors():
        if neighbor.GetSymbol() == 'C':
            length = count_carbon_chain(neighbor, visited)
            max_length = max(max_length, length + 1)
            
    return max_length


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24402',
                          'name': 'glycosphingolipid',
                          'definition': 'A glycosphingolipid is a glycolipid '
                                        'that is a carbohydrate-containing '
                                        'derivative of a sphingoid or '
                                        'ceramide. It is understood that the '
                                        'carbohydrate residue is attached by a '
                                        'glycosidic linkage to O-1 of the '
                                        'sphingoid.',
                          'parents': ['CHEBI:26739', 'CHEBI:33563']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 100,
    'num_false_positives': 100,
    'num_true_negatives': 10845,
    'num_false_negatives': 21,
    'num_negatives': None,
    'precision': 0.5,
    'recall': 0.8264462809917356,
    'f1': 0.6230529595015577,
    'accuracy': 0.989065606361829}


"""
Classifies: CHEBI:133449 hydroxy-fatty acyl-L-carnitine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_hydroxy_fatty_acyl_L_carnitine(smiles: str):
    """
    Determines if a molecule is a hydroxy-fatty acyl-L-carnitine.

    A hydroxy-fatty acyl-L-carnitine is an O-acylcarnitine in which the R group
    is a hydroxylated fatty acyl chain.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxy-fatty acyl-L-carnitine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if molecule contains carnitine moiety
    carnitine_smarts = "[N+](C)(C)C[C@@H](CC([O-])=O)"
    carnitine_match = mol.GetSubstructMatches(Chem.MolFromSmarts(carnitine_smarts))
    if not carnitine_match:
        return False, "Molecule does not contain the carnitine moiety"

    # Find the acyl chain attached to the carnitine
    acyl_chain_atom_idx = mol.GetSubstructMatch(Chem.MolFromSmarts("OC(=O)"))[-1]
    if acyl_chain_atom_idx == -1:
        return False, "No acyl chain found"

    acyl_chain = Chem.Mol(mol.GetTopologicalFragmentAtomIndices(acyl_chain_atom_idx, includeonly=True))

    # Check if the acyl chain contains a hydroxyl group
    hydroxyl_smarts = "[OH]"
    hydroxyl_match = acyl_chain.GetSubstructMatches(Chem.MolFromSmarts(hydroxyl_smarts))
    if not hydroxyl_match:
        return False, "Acyl chain does not contain a hydroxyl group"

    # Check if the acyl chain is a fatty acyl chain
    fatty_acyl_smarts = "[CX3](=O)[C]"
    fatty_acyl_match = acyl_chain.GetSubstructMatches(Chem.MolFromSmarts(fatty_acyl_smarts))
    if not fatty_acyl_match:
        return False, "Acyl chain is not a fatty acyl chain"

    return True, "Molecule is a hydroxy-fatty acyl-L-carnitine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:133449',
                          'name': 'hydroxy-fatty acyl-L-carnitine',
                          'definition': 'An O-acylcarnitine in which the R '
                                        'group is a hydroxylated fatty acyl '
                                        'chain.',
                          'parents': ['CHEBI:176910', 'CHEBI:75659']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "'Mol' object has no attribute "
             "'GetTopologicalFragmentAtomIndices'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:33272 ketimine
"""
from rdkit import Chem

def is_ketimine(smiles: str):
    """
    Determines if a molecule is a ketimine (any imine derived from a ketone).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a ketimine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find imine bonds
    imine_bonds = []
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.DOUBLE:
            begin_atom = bond.GetBeginAtom()
            end_atom = bond.GetEndAtom()
            if begin_atom.GetAtomicNum() == 7 and end_atom.GetAtomicNum() == 6:
                imine_bonds.append(bond)
            elif begin_atom.GetAtomicNum() == 6 and end_atom.GetAtomicNum() == 7:
                imine_bonds.append(bond)

    if not imine_bonds:
        return False, "No imine bonds found"

    # Check if the imine is derived from a ketone
    for bond in imine_bonds:
        carbon_atom = bond.GetBeginAtom() if bond.GetBeginAtom().GetAtomicNum() == 6 else bond.GetEndAtom()
        neighbors = carbon_atom.GetNeighbors()
        carbon_neighbors = [neighbor for neighbor in neighbors if neighbor.GetAtomicNum() == 6]

        if len(carbon_neighbors) == 2:
            for neighbor in carbon_neighbors:
                neighbor_bonds = [bond for bond in neighbor.GetBonds() if bond.GetBeginAtomIdx() == neighbor.GetIdx() or bond.GetEndAtomIdx() == neighbor.GetIdx()]
                double_bond_count = sum(1 for bond in neighbor_bonds if bond.GetBondType() == Chem.BondType.DOUBLE)
                if double_bond_count == 1:
                    return True, "Ketimine derived from a ketone"

    return False, "Not a ketimine or not derived from a ketone"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33272',
                          'name': 'ketimine',
                          'definition': 'Any imine derived from a ketone.',
                          'parents': ['CHEBI:24783']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: '
               "[('C=12N3C(=CC4=[N+]5C(=CC=6N7C=8C(=C9[N+](=C(C1)[C@H]([C@@H]9CCC([O-])=O)C)[Mg-2]753)CC(C8C6C)=O)C(=C4C)CC)C(=C2C)C=C', "
               "'Ketimine derived from a ketone'), ('N1=C(/C=C/CCCCCCC)CCCC1', "
               "'Ketimine derived from a ketone'), "
               "('C1C(N=C2C=C(NN2C1C(F)(F)F)C(=O)O)C3=CC=CO3', 'Ketimine "
               "derived from a ketone'), "
               "('CC1=CC=C(O1)C2=CC(=NN2)C(=O)NN=CC=CC3=CC=CC=C3OC', 'Ketimine "
               "derived from a ketone'), "
               "('C1=CC(=CC=C1C2=CC=C(C=C2)NN=C3C=CC(=O)C(=C3)C(=O)O)NN=C4C=CC(=O)C(=C4)COO', "
               "'Ketimine derived from a ketone'), "
               "('ON=C1C=CC(C=C1)=NNC(=O)c1ccccc1', 'Ketimine derived from a "
               "ketone'), "
               "('OC(=O)CCC=1C2N=C(CC3N=C(CC4=NC(CC=5NC(C2)=C(C5CCC(O)=O)C)C(=C4C)CCC(O)=O)C(=C3C=C)C)C1C', "
               "'Ketimine derived from a ketone'), "
               "('[O-]N=C1C(=O)C=Cc2cc(ccc12)S([O-])(=O)=O', 'Ketimine derived "
               "from a ketone'), "
               "('O=C(NC(C(=O)O)C(O)C(=O)N)/C(=C/C=C/C=C/C=C/C(=C/[C@H]1N=C(/C=C/C=2NC=CC2)O[C@@H]1C)/C)/C', "
               "'Ketimine derived from a ketone'), "
               "('COC(=O)NN=CC1=CC(=NNC2=CC=CC=C2Cl)C=CC1=O', 'Ketimine "
               "derived from a ketone'), ('OC(=O)CC1=NCN=C1', 'Ketimine "
               "derived from a ketone'), "
               "('CCC1=C(C)\\\\C(NC1=O)=C\\\\C1=C(C)C(CCC([O-])=O)=C(N1)\\\\C=C1/N=C(/C=C2\\\\NC(=O)[C@H](C)\\\\C\\\\2=C/C)C(C)=C1CCC([O-])=O', "
               "'Ketimine derived from a ketone'), "
               "('CCOC(=O)C1=C(C(=C2CSC(=NN2)NC3=CC=CC=C3)N=C1C)C', 'Ketimine "
               "derived from a ketone'), "
               "('OC[C@H]1O[C@@H](Oc2cc3C[C@@H](C([O-])=O)\\\\[N+](=C/C=C4C[C@H](NC(=C\\\\4)C(O)=O)C(O)=O)c3cc2O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Ketimine derived from a ketone'), "
               "('C1CN=C(C(=O)N1)CC(=O)C2=CC=C(C=C2)Br', 'Ketimine derived "
               "from a ketone'), "
               "('C1=C2[N+]3=C([C@H](C2(C)C)CCC(N)=O)C(=C2N4[C@@H]5[C@@H]([C@@]2(C)CCC(=O)NC[C@@H](C)OP([O-])(=O)O[C@@H]2[C@@H](CO)O[C@@H]([C@@H]2O)n2c6c(ncnc6[n+]([Co-3]343(C#N)[N+]4=C1[C@H]([C@](C4=C(C1=[N+]3[C@]5(C)[C@@]([C@@H]1CCC(N)=O)(C)CC(N)=O)C)(C)CC(N)=O)CCC(N)=O)c2)N)CC(N)=O)C', "
               "'Ketimine derived from a ketone'), "
               "('OC=1\\\\C(=C/2\\\\C=3C(N=C4C3C=CC=C4)=C(OC)C=C2C)\\\\C(=C(O)C=5C1N=C6C5C=CC=C6)C', "
               "'Ketimine derived from a ketone'), "
               "('C1=2N3C(C=C4N5=C(C=C6N7C(=CC8=N(C(=C1)C(=C8CCC(O)=O)C)[Fe]735)C(=C6C)CCC(O)=O)C(=C4C)C=C)=C(C2C)[C@H](CC/C=C(\\\\C)/CC/C=C(/CCC=C(C)C)\\\\C)O', "
               "'Ketimine derived from a ketone'), "
               "('O=C(OC)[C@@]1(O)C(O)=C2C(=C3C=C4N=C(C=C5N=C(C=C6NC(=C1C2=N3)[C@H]([C@@H]6C)CCC(=O)OC/C=C(/CCC[C@@H](CCC[C@@H](CCCC(C)C)C)C)\\\\C)C(=C5C=C)C)C(=C4CC)C)C', "
               "'Ketimine derived from a ketone'), "
               "('CO\\\\N=C(/C(=O)N[C@H]1[C@H]2SCC(COC(N)=O)=C(N2C1=O)C(=O)OC(C)OC(C)=O)c1ccco1', "
               "'Ketimine derived from a ketone'), "
               "('N1(CC(NC1=O)=O)N=CC=CC=2OC(=CC2)[N+]([O-])=O', 'Ketimine "
               "derived from a ketone'), "
               "('C1(=CC=NC=C1)C(N/N=C(/C([O-])=O)\\\\C)=O.[Ca+2].C1(=CC=NC=C1)C(N/N=C(/C([O-])=O)\\\\C)=O', "
               "'Ketimine derived from a ketone'), "
               "('C=12N3C(=CC4=[N+]5C(=CC=6N7C=8C(=C9[N+](=C(C1)[C@H]([C@@H]9CCC(OC/C=C(/CCC[C@@H](CCC[C@@H](CCCC(C)C)C)C)\\\\C)=O)C)[Mg-2]735)[C-](C(C8C6C)=O)C(=O)OC)[C@@H]([C@H]4C)CC)C(=C2C)C(=O)C', "
               "'Ketimine derived from a ketone'), "
               "('O=C1C(N=C2C(=O)C=3C=4C(=C(C)C=C(C4C2=O)O)C=5O[C@@H](C(C5C3O)(C)C)C)=C(O)C=6C(O)=CC(=C7C6C1=C(O)C=8C([C@@H](C)OC78)(C)C)C', "
               "'Ketimine derived from a ketone'), "
               "('[C@@H]1(N2C(N=C(C(=C2*)*)*)=*)O[C@H](COP(=O)([O-])*)[C@H](C1)O*', "
               "'Ketimine derived from a ketone'), "
               "('[H]C(=C1N=C(CC2NC(=O)C(CC)C2C)C(C)=C1CCC(O)=O)c1[nH]c(CC2NC(=O)C(C)C2CC)c(C)c1CCC(O)=O', "
               "'Ketimine derived from a ketone'), "
               "('O(C=1/C(/N=C(C1)C=2NC=CC2)=C/C=3NC(C)=C4C3[C@H](CCC4)C)C', "
               "'Ketimine derived from a ketone'), "
               "('CCCCN=C1C(=CC=C2C1=NC3(N2O)CCCCC3)[N+](=O)[O-]', 'Ketimine "
               "derived from a ketone'), "
               "('CO[C@H]1\\\\C=C\\\\O[C@@]2(C)Oc3c(C)c(O)c4C(=O)C(NC(=O)\\\\C(C)=C/C=C/[C@H](C)[C@H](O)[C@@H](C)[C@@H](O)[C@@H](C)[C@H](OC(C)=O)[C@@H]1C)=C1NC5(CCN(CC5)CC(C)C)N=C1c4c3C2=O', "
               "'Ketimine derived from a ketone'), "
               "('[Na+].CO\\\\N=C(/C(=O)N[C@H]1[C@H]2SCC([C@@H]3CCCO3)=C(N2C1=O)C([O-])=O)c1csc(N)n1', "
               "'Ketimine derived from a ketone'), "
               "('COC1=C(C=C(C=C1)CC(=O)NN=CC=CC2=CC(=CC=C2)[N+](=O)[O-])OC', "
               "'Ketimine derived from a ketone'), "
               "('C\\\\C=C1\\\\[C@@H](C)C(=O)N\\\\C\\\\1=C/C1=C(C)C(CCC([O-])=O)=C(N1)\\\\C=C1/N=C(/C=C2\\\\NC(=O)C(C=C)=C2C)C(C)=C1CCC([O-])=O', "
               "'Ketimine derived from a ketone'), "
               "('O(C=1C(N=C(C1)C=2NC=CC2)=CC=3NC(C)=C(C3)CCCCC(C)C)C', "
               "'Ketimine derived from a ketone'), "
               "('C=12[C@H]([C@@](CC([O-])=O)(C)C=3[N+]1[Ni-2]45N6C(C3)=C(C(CCC([O-])=O)=C6C=C7[N+]5=C(C=C8N4C(=C2)[C@]([C@@H]8CCC([O-])=O)(CC([O-])=O)C)C(=C7CCC([O-])=O)CC([O-])=O)CC([O-])=O)CCC([O-])=O', "
               "'Ketimine derived from a ketone'), "
               "('O=C(O)C(N=C1C(OC)=C(N)CC(C1)(O)COC2C(O)C(O)C(O)OC2CO)C(OC3C(O)C(O)C(O)OC3CO)C', "
               "'Ketimine derived from a ketone'), "
               "('CC1=C2[N+]3=C(C=C4N5C(=CC6=[N+]7C(Cc8c(CCC([O-])=O)c(CC([O-])=O)c1n8[Co--]357)=C(CCC([O-])=O)C6CC([O-])=O)[C@@H](CCC([O-])=O)[C@]4(C)CC([O-])=O)[C@@H](CCC([O-])=O)[C@]2(C)CC([O-])=O', "
               "'Ketimine derived from a ketone'), "
               "('O\\\\N=C1/Cc2cc(Br)c(Oc3cc(C\\\\C(C(=O)NCCc4ccc(Br)c(Oc5ccc(CCNC1=O)c(Br)c5O)c4)=N/O)cc(Br)c3O)c(Br)c2', "
               "'Ketimine derived from a ketone'), "
               "('CC1=C2[N+]3=C(C=C4N5C(=CC6=[N+]7C(=Cc8c(CCC(O)=O)c(CC(O)=O)c1n8[Co--]357)C(CCC(O)=O)=C6CC(O)=O)[C@@H](CCC(O)=O)[C@]4(C)CC(O)=O)[C@@H](CCC(O)=O)[C@]2(C)CC(O)=O', "
               "'Ketimine derived from a ketone'), "
               "('C(CCCCCCCCCC(O[C@@H]1CC=2[C@]([C@]3(CC[C@]4([C@]([C@@]3(CC2)[H])(CC[C@@]4([C@H](C)CCCC(C)C)[H])[H])C)[H])(C)CC1)=O)C=5N6[B-]([N+]7=C(C=CC7=CC6=CC5)C=8C=CC(=CC8)OC)(F)F', "
               "'Ketimine derived from a ketone'), "
               "('ClC=1C2=C(C(=O)N(C1C)/N=C(/C(=O)C)\\\\C)C(O)=C3C4=C(O)C=5C(=O)C6=C([C@H](OC)[C@@H](O)C[C@H]6OC)OC5C7=C4[C@@H](CC3=C2)OCO7', "
               "'Ketimine derived from a ketone'), "
               "('P(OCC(OCC)COCCCCCCCCCCCCCCCC)(OCC[N+]1=CC(=O)C(N(C)C)C=C1)[O-]', "
               "'Ketimine derived from a ketone'), "
               "('S1C(=NC(=C1)C(=O)NC(C(=O)NC(C(=O)N)=C)=C)C2=N[C@@H]3C=4N=C([C@@H]5NC(=O)C=6N=C([C@H](NC(=O)[C@H]7N=C(\\\\C(\\\\NC([C@@H](NC(C=8N=C([C@@]3(NC(=O)[C@H](NC(=O)C(NC(=O)C(NC(=O)[C@@H](C(C)C)N[C@H]9[C@@H](C=%10N=C(C(O[C@H]5C)=O)C=C(CO)C%10C=C9)O)=C)=C)C)CC2)SC8)=O)[C@@H](O)C)=O)=C\\\\C)SC7)[C@@](O)([C@@H](O)C)C)SC6)SC4', "
               "'Ketimine derived from a ketone'), ('N=C1C=CC(=O)C=C1', "
               "'Ketimine derived from a ketone'), "
               "('OC(=O)[C@@H]1N=C1\\\\C=C\\\\CCCCCCCCCC=C(Br)Br', 'Ketimine "
               "derived from a ketone'), "
               "('C=12N3C(=CC4=[N+]5C(=CC=6N7C=8C(=C9[N+](=C(C1)[C@H]([C@@H]9CCC(O)=O)C)[Mg-2]753)CC(C8C6*)=O)C(=C4C(O)O)C*)C(=C2C)C(O)C', "
               "'Ketimine derived from a ketone'), "
               "('C=12N3C(=CC4=[N+]5C(=CC=6N7C=8C(=C9[N+](=C(C1)[C@H]([C@@H]9CCC(O)=O)C)[Mg-2]735)[C@H](C(C8C6C)=O)C(=O)OC)C(=C4C)CC)C(=C2C)C(=O)C', "
               "'Ketimine derived from a ketone'), "
               "('[C@H](O)(CNC1=C(C(NC(N1)=O)=O)/N=C/C(=O)C)[C@@H]([C@@H](C)O)O', "
               "'Ketimine derived from a ketone'), "
               "('C1=CC(=CC(=C1)Cl)NN=C2C(=NN(C2=O)C(=S)N)C3=CC=C(C=C3)[N+](=O)[O-]', "
               "'Ketimine derived from a ketone'), "
               "('O1[C@@H]([C@H]([C@@H]([C@H]1OCCON=C(C(=O)N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@H](C(N[C@@H](CC(N)=O)C(N2[C@H](C(=O)N)CCC2)=O)=O)C(C)C)=O)C(C)C)=O)CC=3C=CC(=CC3)O)=O)[C@H](O)C)=O)[C@H](O)C)[H])O)O)CO[C@H]4O[C@@H]([C@H]([C@@H]4O)O[C@H]5O[C@@H]([C@H]([C@@H]5O[C@@H]6O[C@@H]([C@H]([C@@H]6O)O)CO)O)CO)CO[C@H]7O[C@@H]([C@H]([C@@H]7O[C@@H]8O[C@@H]([C@H]([C@@H]8O)O)CO)O)CO', "
               "'Ketimine derived from a ketone'), "
               "('C=12N3C(=CC4=[N+]5C(=CC=6N7C=8C(=C9[N+](=C(C1)[C@H]([C@@H]9CCC([O-])=O)C)[Mg-2]753)CC(C8C6CC)=O)C(=C4C)CC(C)C)C(=C2C)C=C', "
               "'Ketimine derived from a ketone'), "
               "('COCCCN=C1C(C(=O)C2=CC=CC=C2C1=O)Cl', 'Ketimine derived from "
               "a ketone'), "
               "('C1CCC(CC1)N=C2C(=CC=C3C2=NC4(N3O)CCCCC4)[N+](=O)[O-]', "
               "'Ketimine derived from a ketone'), "
               "('N1=C(C[C@@H](C1)O)C(O)=O', 'Ketimine derived from a "
               "ketone'), "
               "('O=C1N=CC(C2=CC=C(O)C=C2)=C3C1[C@H](O)[C@](C[C@H](C/C=C/C)C)(C)O3', "
               "'Ketimine derived from a ketone'), "
               "('Cc1c(C=C)c2C=C3[N+]4=C(C=c5c(CCC([O-])=O)c(C)c6=CC7=[N+]8C(=Cc1n2[Fe--]48n56)C(C=C)=C7C)[C@](O)(CCC([O-])=O)[C@@]3(C)O', "
               "'Ketimine derived from a ketone'), "
               "('O=C1NC(O)=C(C=C2C3=C(C=CC(=C3)CC=C(C)C)N=C2C(C=C)(C)C)NC1=O', "
               "'Ketimine derived from a ketone'), "
               "('CC1=NNC(=NC1=O)NN=C(C)C(=O)O', 'Ketimine derived from a "
               "ketone'), "
               "('O=C1NC(O)=C(C=C2C3=C(C=CC(=C3)C[C@H](O)C(O)(C)C)N=C2C(C=C)(C)C)NC1=O', "
               "'Ketimine derived from a ketone'), "
               "('C1[C@H](N=C(C(C1)(C)C)C(=O)[O-])C(=O)[O-]', 'Ketimine "
               "derived from a ketone'), "
               "('N[C@@H](CC1=C/[N+](=C\\\\C=C2C[C@H](NC(=C/2)C(O)=O)C(O)=O)/C=N1)C([O-])=O', "
               "'Ketimine derived from a ketone'), "
               "('CN1C2=NC3(CCCCC3)ON=C2C(=O)N(C1=O)C', 'Ketimine derived from "
               "a ketone'), "
               "('C1OC2=C(O1)C=C(C=C2)NN=C3C(=NN(C3=N)C4=CC=CC=C4)N', "
               "'Ketimine derived from a ketone'), "
               "('O=C1N(C(O)=C(C=C2C3=C(C=CC=C3)N=C2C(C=C)(C)C)NC1=O)C', "
               "'Ketimine derived from a ketone'), "
               "('O=C1OC=2C=3C(C(O)=C4C2C5(O)C(=O)CC(C)CC5(O[C@H]6O[C@@H]([C@H](O[C@H]7O[C@@H]([C@H](O)[C@@H](C7)O)C)[C@@H](C6)N(C)C)C)C=C4)=C(O)C(=CC3C1=C8C9=C(C=CC=C9)N=C8)[C@@H]%10O[C@@H]([C@@H](O)[C@@H](C%10)O)C', "
               "'Ketimine derived from a ketone'), "
               "('C[C@]1(CC(O)=O)[C@H](CCC(O)=O)C2=CC3=[N+]4C(Cc5c(CCC(O)=O)c(CC(O)=O)c6C=C7[N+]8=C(C=C1N2[Co--]48n56)[C@@H](CCC(O)=O)[C@]7(C)CC(O)=O)=C(CCC(O)=O)C3CC(O)=O', "
               "'Ketimine derived from a ketone'), "
               "('C/1(\\\\C(N(N=C1C2=CC=CC=C2)C3=CC=CC(=C3)C(O)=O)=O)=C\\\\C4=CC=C(C=C4)C5=CC=CC=C5', "
               "'Ketimine derived from a ketone'), "
               "('O(C1=C(NC(=C1)C=2NC=CC2)C3=C4N=C5CCCCCCCCC3C(C(C)C)C4=C5)C', "
               "'Ketimine derived from a ketone'), "
               "('C1=CC=CC(=C1)C=2OC(=CC2)/C=C/3\\\\C(N(N=C3C4=CC=CC=C4)C5=CC=CC(=C5)C(O)=O)=O', "
               "'Ketimine derived from a ketone'), "
               "('CC1=C(CCC(O)=O)\\\\C(/N=C1/C=C1NC(=O)C(C=C)=C/1C)=C\\\\c1[nH]c(\\\\C=C2NC(=O)C(C)=C/2C=C)c(C)c1CCC(O)=O', "
               "'Ketimine derived from a ketone'), "
               "('O(C1=C(NC(=C1)C=2NC=CC2)C(C=3NC=4CCCCCC[C@@H](C3C4)CCCC)=C5N=C(CCCCCCCCCCC)C=C5)C', "
               "'Ketimine derived from a ketone'), "
               "('O=NN/N=C/C=C/C=C/C=C/CCCC', 'Ketimine derived from a "
               "ketone'), ('C1C=CC=C2Sc3ccccc3N=C12', 'Ketimine derived from a "
               "ketone'), "
               "('N=1C2=C(C=CC=C2)C(C1)=C(C=3C4=C(C=CC=C4)NC3)C=5C6=C(C=CC=C6)NC5', "
               "'Ketimine derived from a ketone'), "
               "('CCCCCCCCCCCC1=CC=C(N1)\\\\C=C1/N=C(C=C1OC)C1=CC=CN1', "
               "'Ketimine derived from a ketone'), "
               "('COC(=O)[C@H]1C(=O)c2c(C)c3=CC4=N\\\\C(=C/c5c(C(C)=O)c(C)c6\\\\C=C7/N=C([C@@H](CCC(=O)OC\\\\C=C(/C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C)[C@@H]7C)C1=c2n3[Mg]n56)[C@H](C)C/4=C/C', "
               "'Ketimine derived from a ketone'), "
               "('C1=CC=C(C=C1)C2=NN=CC2=CNNC(=O)C3=CC=CS3', 'Ketimine derived "
               "from a ketone'), "
               "('O=C(NC(C(=O)O)CC(=O)N)/C(=C/C=C/C=C/C=C/C=C/C=C/C(=C/[C@H]1N=C(/C=C/C=2NC=CC2)O[C@@H]1C)/C)/C', "
               "'Ketimine derived from a ketone'), "
               "('C1=2N3C(C=C4N5=C(C=C6N7C(=CC8=N(C(=C1)C(=C8CCC(O)=O)C)[Fe+]573)C(=C6C)CCC(O)=O)C(=C4C)C=C)=C(C2C)C=C', "
               "'Ketimine derived from a ketone'), "
               "('C1=CC=C2C(=C1)C=CC(=C2O)NN=C3C4=C(C=C(C=C4)[N+](=O)[O-])C(=CC3=O)S(=O)(=O)O', "
               "'Ketimine derived from a ketone'), ('O=C1N=CC=N1', 'Ketimine "
               "derived from a ketone'), "
               "('O(C(=O)CCC1C(C=2NC1=C3CC(=O)C=4C3=NC(C4C)=CC=5N=C(C=C6N=C(C2)C(=C6C=C)C)C(C5CC)=CO)C)C\\\\C=C(\\\\CC(CC(CCCC(CCCCC)C)C)C)/C', "
               "'Ketimine derived from a ketone'), "
               "('CC(=O)NC1=CC=C(C=C1)S(=O)(=O)NC2=C(NC3=CC=CC=C3N2)N=C4C=CC(=O)C(=C4)C(=O)O', "
               "'Ketimine derived from a ketone'), "
               "('C=12N3C(=CC4=[N+]5C(=CC=6N7C=8C(=C9[N+](=C(C1)[C@H]([C@@H]9CCC([O-])=O)C)[Mg-2]753)CC(C8C6*)=O)C(=C4C=O)C*)C(=C2C)C(O)C', "
               "'Ketimine derived from a ketone'), "
               "('O=C1NC(O)=C(C=C2C3=C(C=CC=C3)N=C2C(C=C)(C)C)NC1=O', "
               "'Ketimine derived from a ketone'), "
               "('C1CN(CCN1C2=CC=CC=N2)N=CC=CC3=CC=CC=C3', 'Ketimine derived "
               "from a ketone'), "
               "('COC1=C(Cc2c[nH]c3c(CC=C(C)C)cccc23)N2O[Fe-3]34(ON5C(Cc6c[nH]c7c(CC=C(C)C)cccc67)=C(OC)N=C(C)C5=[O+]3)(ON3C(Cc5c[nH]c6c(CC=C(C)C)cccc56)=C(OC)N=C(C)C3=[O+]4)[O+]=C2C(C)=N1', "
               "'Ketimine derived from a ketone'), "
               "('C=12N3C(=CC4=NC(=CC=5N(C=6C(=C7N=C(C1)[C@H]([C@@H]7CCC(O)=O)C)[C@H](C(C6C5C)=O)C(=O)OC)[Mg]3)C(=C4CO)CC)C(=C2C)C=C', "
               "'Ketimine derived from a ketone'), "
               "('O=C(NC(O)(C)C)C1C2(C(C3C(C4(C(N=C(O)C=C4)CC3)C)CC2)CC1)C', "
               "'Ketimine derived from a ketone'), "
               "('C=1(NC(/C=C/2\\\\NC(C(=C2C)C=C)=O)=C(C1C)C=C)/C=C/3\\\\N=C(\\\\C=C/4\\\\C(=C(C(N4)=O)C)CCC(O)=O)C(=C3C)CCC(O)=O', "
               "'Ketimine derived from a ketone'), "
               "('Nc1ccc(cc1)\\\\N=C1/C=C(N)\\\\C(\\\\C=C/1N)=N\\\\c1ccc(N)cc1', "
               "'Ketimine derived from a ketone'), ('C1=CC2N=c3ccccc3=C2C=C1', "
               "'Ketimine derived from a ketone'), "
               "('O=C(O)C1=C(N/N=C(\\\\C(=O)C)/C)C=CC=C1', 'Ketimine derived "
               "from a ketone'), "
               "('CCC1=C(C)\\\\C(NC1=O)=C\\\\c1[nH]c(\\\\C=C2/N=C(/C=C3\\\\NC(=O)[C@H](C)C3=CC)C(C)=C/2CCC(O)=O)c(CCC(O)=O)c1C', "
               "'Ketimine derived from a ketone'), "
               "('C1=CC(=CC(=C1C(=O)O)C=2OC(=CC2)/C=C/3\\\\C(N(N=C3C4=CC=CC=C4)C5=CC=C(C=C5)C(C)C)=O)Cl', "
               "'Ketimine derived from a ketone'), "
               "('O(C=1C(N=C(C1)C=2NC=CC2)=CC=3NC(C=4NC=CC4)=CC3OC)C', "
               "'Ketimine derived from a ketone'), "
               "('C1=2N3C(C=C4N5=C(C=C6N7C(=CC8=N(C(=C1)C(=C8CCC([O-])=O)C([H])=O)[Fe+]735)C(=C6C)CCC([O-])=O)C(=C4C)C=C)=C(C2C)[C@H](CCCC(C)CCCC(CCCC(C)C)C)O', "
               "'Ketimine derived from a ketone'), "
               "('CCC1=C(C)C(=O)N[C@H]1Cc1[nH]c(\\\\C=C2/N=C(C[C@H]3NC(=O)C(CC)=C3C)C(C)=C/2CCC(O)=O)c(CCC(O)=O)c1C', "
               "'Ketimine derived from a ketone'), "
               "('C=12[N+]3=C(C=C4N5C(=CC6=[N+]7C(=CC=8N(C(C1)=C(C8CCC([O-])=O)C)[Fe-2]537)C(=C6C)CCC(=O)[O-])[C@H]([C@@]4(CC(=O)[O-])C)CCC([O-])=O)[C@H]([C@@]2(CC([O-])=O)C)CCC([O-])=O', "
               "'Ketimine derived from a ketone'), "
               "('O=C1N[C@@](ON=C1[C@H](CC)C)(C(=O)OCC)[C@H](CC)C', 'Ketimine "
               "derived from a ketone'), "
               "('C1(NC(/C=C/2\\\\N=C(/C=C/3\\\\N\\\\C(=C/C4=NC(C(=C4CCC(O)=O)C)=O)\\\\C(=C3C)CCC(O)=O)C(=C2C)C=C)=C(C1C)C=C)=O', "
               "'Ketimine derived from a ketone')]\n"
               "False negatives: [('CC(=N)C(O)=O', 'No imine bonds found'), "
               "('C(=O)(C(=N)C[C@H](O)[C@H](O)CO)O', 'No imine bonds found')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 12403,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9920031987205118}


"""
Classifies: CHEBI:140324 primary carboxamide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_primary_carboxamide(smiles: str):
    """
    Determines if a molecule contains a primary carboxamide group (RC(=O)NH2).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule contains primary carboxamide, False otherwise
        str: Reason for classification
    """
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # SMARTS pattern for primary carboxamide: RC(=O)NH2
    # [CX3](=[OX1])[NX3H2]
    # CX3 = carbon with 3 bonds
    # =OX1 = oxygen double bond
    # NX3H2 = nitrogen with 3 bonds and 2 hydrogens
    pattern = Chem.MolFromSmarts('[CX3](=[OX1])[NX3H2]')
    
    if mol.HasSubstructMatch(pattern):
        # Get number of matches
        matches = mol.GetSubstructMatches(pattern)
        
        if len(matches) == 1:
            return True, "Contains one primary carboxamide group"
        else:
            return True, f"Contains {len(matches)} primary carboxamide groups"
            
    return False, "No primary carboxamide group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140324',
                          'name': 'primary carboxamide',
                          'definition': 'A carboxamide resulting from the '
                                        'formal condensation of a carboxylic '
                                        'acid with ammonia; formula RC(=O)NH2.',
                          'parents': ['CHEBI:37622']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 10,
    'num_false_positives': 100,
    'num_true_negatives': 2674,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.09090909090909091,
    'recall': 1.0,
    'f1': 0.16666666666666669,
    'accuracy': 0.9640804597701149}


"""
Classifies: CHEBI:23016 carbonates
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_carbonates(smiles: str):
    """
    Determines if a molecule contains a carbonate group (salt or ester of carbonic acid).
    Carbonate pattern: R-O-C(=O)-O-R or R-O-C(=O)-O-M+ where R is organic group and M is metal

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains carbonate group, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
    
    # SMARTS patterns for carbonate groups
    # Organic carbonate ester pattern: R-O-C(=O)-O-R
    carbonate_ester_pattern = Chem.MolFromSmarts('[C;!$(C=[!O])]-O-C(=O)-O-[C;!$(C=[!O])]')
    # Mixed carbonate pattern: R-O-C(=O)-O-R where R can be C or H
    mixed_carbonate_pattern = Chem.MolFromSmarts('[#6,#1]-O-C(=O)-O-[#6,#1]')
    # Inorganic carbonate pattern: [O-]C(=O)[O-]
    inorganic_carbonate_pattern = Chem.MolFromSmarts('[O-]C(=O)[O-]')
    
    # Check for organic carbonate esters
    if mol.HasSubstructMatch(carbonate_ester_pattern):
        matches = mol.GetSubstructMatches(carbonate_ester_pattern)
        return True, f"Contains organic carbonate ester group(s) ({len(matches)} found)"
        
    # Check for mixed carbonates
    if mol.HasSubstructMatch(mixed_carbonate_pattern):
        matches = mol.GetSubstructMatches(mixed_carbonate_pattern)
        return True, f"Contains mixed carbonate group(s) ({len(matches)} found)"
        
    # Check for inorganic carbonates
    if mol.HasSubstructMatch(inorganic_carbonate_pattern):
        matches = mol.GetSubstructMatches(inorganic_carbonate_pattern)
        return True, f"Contains inorganic carbonate group(s) ({len(matches)} found)"
    
    # Additional check for carbonate esters with specific patterns
    methyl_carbonate = Chem.MolFromSmarts('COC(=O)O[#6]')
    ethyl_carbonate = Chem.MolFromSmarts('CCO-C(=O)O[#6]')
    
    if mol.HasSubstructMatch(methyl_carbonate):
        return True, "Contains methyl carbonate ester group"
    if mol.HasSubstructMatch(ethyl_carbonate):
        return True, "Contains ethyl carbonate ester group"
        
    return False, "No carbonate groups found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23016',
                          'name': 'carbonates',
                          'definition': 'Organooxygen compounds that are salts '
                                        'or esters of carbonic acid, H2CO3.',
                          'parents': ['CHEBI:36963']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 72,
    'num_true_negatives': 183822,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.04,
    'recall': 0.75,
    'f1': 0.07594936708860758,
    'accuracy': 0.9996030408161046}


"""
Classifies: CHEBI:33702 polyatomic cation
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_polyatomic_cation(smiles: str):
    """
    Determines if a molecule is a polyatomic cation (a cation consisting of more than one atom).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a polyatomic cation, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule has a formal charge
    charge = AllChem.GetFormalCharge(mol)
    if charge <= 0:
        return False, "Molecule does not have a positive formal charge"

    # Check if the molecule consists of more than one atom
    num_atoms = mol.GetNumAtoms()
    if num_atoms <= 1:
        return False, "Molecule consists of only one atom"

    # If the molecule has a positive charge and more than one atom, it is a polyatomic cation
    return True, "Polyatomic cation"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33702',
                          'name': 'polyatomic cation',
                          'definition': 'A cation consisting of more than one '
                                        'atom.',
                          'parents': ['CHEBI:36358', 'CHEBI:36916']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 53,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 100,
    'num_negatives': None,
    'precision': 1.0,
    'recall': 0.3464052287581699,
    'f1': 0.5145631067961165,
    'accuracy': 0.3464052287581699}


"""
Classifies: CHEBI:26907 tetrahydrofolic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_tetrahydrofolic_acid(smiles: str) -> tuple[bool, str]:
    """
    Determines if a molecule is a tetrahydrofolic acid, which is defined as a group of heterocyclic compounds
    based on the 5,6,7,8-tetrahydropteroic acid skeleton conjugated with one or more L-glutamic acid units.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a tetrahydrofolic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains a tetrahydropteroic acid skeleton
    skeleton_smarts = '[#7]1([#6]([#7])([#6]([#7])([#6]([#7])([#6]1)))([#6]([#6]1=[#6]([#7])([#6]([#6]1)[#7])([#6]([#6]([#6]([#6]([#6]([#7])([#6]1=[#7]))([#6]1=[#7]))([#6]([#7])([#6]([#6]1)([#7])([#6]([#6]([#6]([#6]([#6]([#6])([#6])([#6]=[#6]))([#6]1=[#7]))([#7])([#6]1=[#6]([#7])([#6]([#6]1)[#7])([#6]([#6]([#8])([#6]))([#7]))([#6]=[#8]))([#7])([#6]([#6]([#6]([#8])([#8])([#8]))([#7])([#7]))([#6]([#6]([#6])([#6])([#6]([#8])([#8]))([#7])([#7]))'
    skeleton_match = mol.HasSubstructMatch(Chem.MolFromSmarts(skeleton_smarts))

    if not skeleton_match:
        return False, "Does not contain a tetrahydropteroic acid skeleton"

    # Check if the molecule contains a glutamic acid unit
    glutamic_smarts = '[#6]([#6]([#6]([#8])([#8]))([#7])([#6]([#6]([#8])([#8]))([#7]))'
    glutamic_match = mol.HasSubstructMatch(Chem.MolFromSmarts(glutamic_smarts))

    if not glutamic_match:
        return False, "Does not contain a glutamic acid unit"

    return True, "Molecule is a tetrahydrofolic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26907',
                          'name': 'tetrahydrofolic acid',
                          'definition': 'A group of heterocyclic compounds '
                                        'based on the '
                                        '5,6,7,8-tetrahydropteroic acid '
                                        'skeleton conjugated with one or more '
                                        'L-glutamic acid units.',
                          'parents': ['CHEBI:37445']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.HasSubstructMatch(Mol, NoneType)\n'
             'did not match C++ signature:\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'RDKit::SubstructMatchParameters params=True)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'RDKit::SubstructMatchParameters params)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:12777 vitamin A
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_vitamin_A(smiles: str):
    """
    Determines if a molecule is a vitamin A compound.

    A vitamin A compound is defined as any member of a group of fat-soluble retinoids
    produced via metabolism of provitamin A carotenoids that exhibit biological activity
    against vitamin A deficiency. Vitamin A is involved in immune function, vision,
    reproduction, and cellular communication.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a vitamin A compound, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of the retinoid backbone
    pattern = Chem.MolFromSmarts('[C@@H](C=C[C@H](C)C=C[C@H](C)C=C[C@@H](C)C=C[C@@H](C)C=C[C@@H](C)C=C[C@@H](C)C=C[C@@H](C)C=C[C@@H](C)C=O)')
    if not mol.HasSubstructMatch(pattern):
        return False, "Molecule does not contain the retinoid backbone"

    # Check for the presence of a carboxylic acid group
    pattern = Chem.MolFromSmarts('C(=O)O')
    if not mol.HasSubstructMatch(pattern):
        return False, "Molecule does not contain a carboxylic acid group"

    return True, "Molecule is a vitamin A compound"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:12777',
                          'name': 'vitamin A',
                          'definition': 'Any member of a group of fat-soluble '
                                        'retinoids produced via metabolism of '
                                        'provitamin A carotenoids that exhibit '
                                        'biological activity against vitamin A '
                                        'deficiency. Vitamin A is involved in '
                                        'immune function, vision, '
                                        'reproduction, and cellular '
                                        'communication.',
                          'parents': ['CHEBI:26537']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183924,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630012234}


"""
Classifies: CHEBI:23482 cyclohexanones
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_cyclohexanones(smiles: str):
    """
    Determines if a molecule is a cyclohexanone (cyclic 6-membered ketone) or derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a cyclohexanone, False otherwise
        str: Reason for classification
    """
    # Convert SMILES to molecule
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Get ring information
    rings = mol.GetRingInfo()
    
    # Find 6-membered rings
    six_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            six_rings.append(ring)
            
    if not six_rings:
        return False, "No 6-membered rings found"

    # For each 6-membered ring, check if it contains a ketone
    for ring in six_rings:
        ring_atoms = [mol.GetAtomWithIdx(i) for i in ring]
        
        # Check if ring is aliphatic (not aromatic)
        if any(atom.GetIsAromatic() for atom in ring_atoms):
            continue
            
        # Look for ketone pattern (C(=O)C) in ring
        for atom_idx in ring:
            atom = mol.GetAtomWithIdx(atom_idx)
            
            # Check if atom is carbon
            if atom.GetSymbol() != 'C':
                continue
                
            # Check for double-bonded oxygen
            carbonyl_o = False
            ring_carbons = 0
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'O' and \
                   mol.GetBondBetweenAtoms(atom_idx, neighbor.GetIdx()).GetBondType() == Chem.BondType.DOUBLE:
                    carbonyl_o = True
                # Count carbons that are part of the ring
                if neighbor.GetSymbol() == 'C' and neighbor.GetIdx() in ring:
                    ring_carbons += 1
                    
            # Valid ketone pattern found in ring
            if carbonyl_o and ring_carbons == 2:
                # Get substituents
                substituents = []
                for ring_atom in ring_atoms:
                    for neighbor in ring_atom.GetNeighbors():
                        if neighbor.GetIdx() not in ring:
                            substituents.append(neighbor.GetSymbol())
                
                if len(substituents) > 0:
                    return True, f"Substituted cyclohexanone with substituents: {', '.join(set(substituents))}"
                else:
                    return True, "Unsubstituted cyclohexanone"
                    
    return False, "No cyclohexanone pattern found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23482',
                          'name': 'cyclohexanones',
                          'definition': 'Any alicyclic ketone based on a '
                                        'cyclohexane skeleton and its '
                                        'substituted derivatives thereof.',
                          'parents': ['CHEBI:36132']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 1924,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9507146377525875}


"""
Classifies: CHEBI:28163 iron(III) hydroxamate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_iron_III_hydroxamate(smiles: str):
    """
    Determines if a molecule is an iron(III) hydroxamate complex.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an iron(III) hydroxamate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of iron(III)
    iron_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'Fe' and atom.GetFormalCharge() == 3]
    if not iron_atoms:
        return False, "No iron(III) atom found"

    # Check for the presence of three hydroxamic acid groups
    hydroxamic_acid_count = sum(Chem.MolFromSmarts('C(=O)NO').GetNumAtomsMatchedBySmarts(mol))
    if hydroxamic_acid_count != 3:
        return False, f"Found {hydroxamic_acid_count} hydroxamic acid groups, expected 3"

    # Check if the three hydroxamic acid groups are coordinated to the iron(III)
    iron_atom = iron_atoms[0]
    coordinated_atoms = [atom for atom in iron_atom.GetNeighbors()]
    coordinated_hydroxamic_acids = sum(1 for atom in coordinated_atoms if atom.IsInRingSize(5) and atom.GetSymbol() == 'N')
    if coordinated_hydroxamic_acids != 3:
        return False, f"Found {coordinated_hydroxamic_acids} hydroxamic acid groups coordinated to iron(III), expected 3"

    return True, "Molecule is an iron(III) hydroxamate complex"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28163',
                          'name': 'iron(III) hydroxamate',
                          'definition': 'A complex between iron(III) and three '
                                        'hydroxamic acid groups, used for iron '
                                        'transport.',
                          'parents': ['CHEBI:5975']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "name 'is_iron_III__hydroxamate' is not defined",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:25372 molybdopterin cofactor
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_molybdopterin_cofactor(smiles: str):
    """
    Determines if a molecule is a molybdopterin cofactor.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a molybdopterin cofactor, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a molybdenum atom
    if mol.HasSubstructMatch(Chem.MolFromSmarts("[Mo]")):
        # Check for the presence of a pterin ring
        pterin_ring = Chem.MolFromSmarts("c1nc2[nH]cnc2n[nH]1")
        if mol.HasSubstructMatch(pterin_ring):
            # Check for the presence of sulfur atoms coordinated to molybdenum
            mo_atom = next(atom for atom in mol.GetAtoms() if atom.GetSymbol() == "Mo")
            sulfur_neighbors = [nbr for nbr in mo_atom.GetNeighbors() if nbr.GetSymbol() == "S"]
            if len(sulfur_neighbors) >= 2:
                return True, "Molybdopterin cofactor with molybdenum and sulfur coordination"
            else:
                return False, "Missing sulfur coordination to molybdenum"
        else:
            return False, "Missing pterin ring"
    else:
        return False, "Missing molybdenum atom"

    return False, "Not a molybdopterin cofactor"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25372',
                          'name': 'molybdopterin cofactor',
                          'definition': 'A molybdopterin to which a metal or '
                                        'metal oxide is coordinated, and that '
                                        'acts as a prosthetic group.',
                          'parents': ['CHEBI:35203']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183918,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.9999945628238518}


"""
Classifies: CHEBI:33583 noble gas molecular entity
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_noble_gas_molecular_entity(smiles: str):
    """
    Determines if a molecule is a noble gas molecular entity.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a noble gas molecular entity, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # List of noble gas atomic numbers
    noble_gas_nums = [2, 10, 18, 36, 54, 86]  # He, Ne, Ar, Kr, Xe, Rn

    # Get the atomic numbers of atoms in the molecule
    atom_nums = [atom.GetAtomicNum() for atom in mol.GetAtoms()]

    # Check if all atoms are noble gases
    all_noble_gases = all(num in noble_gas_nums for num in atom_nums)

    if all_noble_gases:
        # Check if the molecule has more than one atom
        if len(atom_nums) > 1:
            noble_gas_symbols = [Chem.GetPeriodicTable().GetElementSymbol(num) for num in atom_nums]
            return True, f"A molecular entity containing noble gas atoms: {', '.join(noble_gas_symbols)}"
        else:
            # Handle cases like [He] or [Rn]
            atom_symbol = Chem.GetPeriodicTable().GetElementSymbol(atom_nums[0])
            return False, f"Not a molecular entity, but a single atom: {atom_symbol}"
    else:
        return False, "Not a noble gas molecular entity"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:33583',
                          'name': 'noble gas molecular entity',
                          'definition': 'A main group molecular entity '
                                        'containing one or more atoms of any '
                                        'noble gas.',
                          'parents': ['CHEBI:33579']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.3076923076923077 is too low.\n'
               "True positives: [('[He][He]', 'A molecular entity containing "
               "only noble gas atoms: He, He'), ('[He][He+]', 'A molecular "
               "entity containing only noble gas atoms: He, He')]\n"
               "False positives: [('[4He]', 'A molecular entity containing "
               "only noble gas atoms: He'), ('[6He]', 'A molecular entity "
               "containing only noble gas atoms: He'), ('[220Rn]', 'A "
               "molecular entity containing only noble gas atoms: Rn'), "
               "('[222Rn]', 'A molecular entity containing only noble gas "
               "atoms: Rn'), ('[3He]', 'A molecular entity containing only "
               "noble gas atoms: He'), ('[8He]', 'A molecular entity "
               "containing only noble gas atoms: He'), ('[219Rn]', 'A "
               "molecular entity containing only noble gas atoms: Rn'), "
               "('[Rn+]', 'A molecular entity containing only noble gas atoms: "
               "Rn'), ('[Rn]', 'A molecular entity containing only noble gas "
               "atoms: Rn')]\n"
               'False negatives: []',
    'attempt': 2,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 0,
    'num_true_negatives': 183911,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 1.0,
    'recall': 1.0,
    'f1': 1.0,
    'accuracy': 1.0}


"""
Classifies: CHEBI:22455 alpha-hydroxynitrile
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_alpha_hydroxynitrile(smiles: str):
    """
    Determines if a molecule is an alpha-hydroxynitrile.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an alpha-hydroxynitrile, False otherwise
        str: Reason for classification
    """
    # Create RDKit mol object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Look for CN triple bond (nitrile group)
    nitrile_pattern = Chem.MolFromSmarts('[C]#N')
    nitrile_matches = mol.GetSubstructMatches(nitrile_pattern)
    
    if not nitrile_matches:
        return False, "No nitrile group found"
        
    # Look for OH group
    hydroxy_pattern = Chem.MolFromSmarts('[OH1]')
    hydroxy_matches = mol.GetSubstructMatches(hydroxy_pattern)
    
    if not hydroxy_matches:
        return False, "No hydroxy group found"
    
    # Check if OH is on carbon alpha to CN
    for nitrile_match in nitrile_matches:
        nitrile_c = nitrile_match[0]  # Get carbon atom index of CN
        
        # Get atom connected to nitrile carbon
        alpha_atom = None
        for atom in mol.GetAtomWithIdx(nitrile_c).GetNeighbors():
            if atom.GetAtomicNum() != 7:  # Not nitrogen
                alpha_atom = atom
                break
                
        if alpha_atom is None:
            continue
            
        # Check if alpha carbon has OH attached
        for neighbor in alpha_atom.GetNeighbors():
            if neighbor.GetAtomicNum() == 8 and neighbor.GetTotalNumHs() == 1:
                return True, "Found hydroxy group on carbon alpha to nitrile"
                
    return False, "Hydroxy group not alpha to nitrile"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22455',
                          'name': 'alpha-hydroxynitrile',
                          'definition': 'A hydroxynitrile in which the hydroxy '
                                        'group is located on the carbon alpha '
                                        'to the carbonitrile group.',
                          'parents': ['CHEBI:24730']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 9,
    'num_true_negatives': 183888,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.25,
    'recall': 1.0,
    'f1': 0.4,
    'accuracy': 0.9999510603588907}


"""
Classifies: CHEBI:27024 toluenes
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_toluene(smiles: str) -> tuple[bool, str]:
    """
    Determines if a molecule is a toluene (a substituted benzene with one and only one methyl group).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a toluene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Generate the aromatic ring information
    rings = mol.GetRingInfo()

    # Check for at least one 6-membered ring
    if not any(len(ring) == 6 for ring in rings.AtomRings()):
        return False, "No 6-membered rings found"

    # Find all aromatic 6-membered rings
    aromatic_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings.append(ring)

    if not aromatic_rings:
        return False, "No aromatic 6-membered rings found"

    # Check if all carbons in the aromatic ring are carbon
    for ring in aromatic_rings:
        atoms = [mol.GetAtomWithIdx(i) for i in ring]
        if not all(atom.GetSymbol() == 'C' for atom in atoms):
            return False, "Ring contains non-carbon atoms"

    # Check for exactly one methyl substituent
    ring_atoms = set(aromatic_rings[0])
    methyl_count = 0

    for atom_idx in ring_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in ring_atoms:
                if neighbor.GetSymbol() == 'C' and neighbor.GetDegree() == 4:
                    methyl_count += 1

    if methyl_count != 1:
        return False, "Number of methyl groups is not equal to 1"

    return True, "Toluene (substituted benzene with one methyl group)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27024',
                          'name': 'toluenes',
                          'definition': 'Any member of the class of benzenes '
                                        'that is a substituted benzene in '
                                        'which the substituents include one '
                                        '(and only one) methyl group.',
                          'parents': ['CHEBI:22712']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183703,
    'num_false_negatives': 23,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9998748135810935}


"""
Classifies: CHEBI:26588 1,3,5-triazines
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_1_3_5_triazines(smiles: str):
    """
    Determines if a molecule contains a 1,3,5-triazine core structure.
    A 1,3,5-triazine has nitrogen atoms at positions 1, 3 and 5 of a 6-membered aromatic ring.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains 1,3,5-triazine core, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # SMARTS pattern for 1,3,5-triazine core
    # n1cnc(nc1) represents a 6-membered ring with alternating N atoms
    triazine_pattern = Chem.MolFromSmarts('n1cnc(nc1)')
    
    if mol.HasSubstructMatch(triazine_pattern):
        # Get matches
        matches = mol.GetSubstructMatches(triazine_pattern)
        
        for match in matches:
            # Get the atoms in the matched ring
            ring_atoms = [mol.GetAtomWithIdx(i) for i in match]
            
            # Count nitrogen atoms in the ring
            n_count = sum(1 for atom in ring_atoms if atom.GetSymbol() == 'N')
            
            # Verify there are exactly 3 nitrogens
            if n_count == 3:
                # Check if the nitrogens are at positions 1,3,5
                n_positions = [i for i, atom in enumerate(ring_atoms) if atom.GetSymbol() == 'N']
                if set(n_positions) == {0, 2, 4}:  # positions 1,3,5 in 0-based indexing
                    # Get substituents
                    substituents = []
                    for atom_idx in match:
                        atom = mol.GetAtomWithIdx(atom_idx)
                        for neighbor in atom.GetNeighbors():
                            if neighbor.GetIdx() not in match:
                                substituents.append(neighbor.GetSymbol())
                    
                    if substituents:
                        return True, f"1,3,5-triazine with substituents: {', '.join(set(substituents))}"
                    else:
                        return True, "Unsubstituted 1,3,5-triazine"
                    
    return False, "No 1,3,5-triazine core structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26588',
                          'name': '1,3,5-triazines',
                          'definition': 'Any compound with a 1,3,5-triazine '
                                        'skeleton, in which nitrogen atoms '
                                        'replace carbon at positions 1, 3 and '
                                        '5 of the core benzene ring structure.',
                          'parents': ['CHEBI:38102']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 12,
    'num_false_positives': 100,
    'num_true_negatives': 150874,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.10714285714285714,
    'recall': 0.9230769230769231,
    'f1': 0.19199999999999998,
    'accuracy': 0.9993310682376628}


"""
Classifies: CHEBI:137967 N(6)-acyl-L-lysine residue
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_N_6__acyl_L_lysine_residue(smiles: str):
    """
    Determines if a molecule is an N(6)-acyl-L-lysine residue, defined as an L-alpha-amino acid residue derived from any N(6)-acyl-L-lysine.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an N(6)-acyl-L-lysine residue, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a lysine residue
    lysine_pattern = Chem.MolFromSmarts('C(C(=O)N)CCC[NH3+]')
    if not mol.HasSubstructMatch(lysine_pattern):
        return False, "Not a lysine residue"

    # Check for acylation at the N(6) position
    acyl_pattern = Chem.MolFromSmarts('C(=O)NCC(C(=O)N)CCC[NH3+]')
    if not mol.HasSubstructMatch(acyl_pattern):
        return False, "Not acylated at the N(6) position"

    # Check for L-configuration at the alpha carbon
    alpha_carbon = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetDegree() == 3:
            neighbors = [mol.GetAtomWithIdx(n) for n in atom.GetNeighbors()]
            if len([n for n in neighbors if n.GetSymbol() == 'N']) == 1 and len([n for n in neighbors if n.GetSymbol() == 'O']) == 1:
                alpha_carbon = atom
                break

    if alpha_carbon is None:
        return False, "Unable to identify alpha carbon"

    if alpha_carbon.GetChiralTag() != Chem.ChiralType.CHI_TETRAHEDRAL_CCW:
        return False, "Not L-configuration at alpha carbon"

    return True, "Molecule is an N(6)-acyl-L-lysine residue"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:137967',
                          'name': 'N(6)-acyl-L-lysine residue',
                          'definition': 'An L-alpha-amino acid residue derived '
                                        'from any N(6)-acyl-L-lysine.',
                          'parents': ['CHEBI:83228']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183918,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945628238518}


"""
Classifies: CHEBI:2580 unsaturated fatty acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_unsaturated_fatty_acid_anion(smiles: str):
    """
    Determines if a molecule is an unsaturated fatty acid anion.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an unsaturated fatty acid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for carboxylate anion
    carboxylate_pattern = Chem.MolFromSmarts('C(=O)[O-]')
    if not mol.HasSubstructMatch(carboxylate_pattern):
        return False, "No carboxylate anion found"
        
    # Count number of carboxylate groups
    carboxylate_matches = mol.GetSubstructMatches(carboxylate_pattern)
    if len(carboxylate_matches) > 1:
        return False, "Multiple carboxylate groups found"

    # Check for carbon chain
    carbon_chain = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            carbon_chain = True
            break
    if not carbon_chain:
        return False, "No carbon chain found"

    # Check for unsaturation (C=C double bonds)
    unsaturation_patterns = [
        Chem.MolFromSmarts('C=C'),  # explicit double bond
        Chem.MolFromSmarts('C/C=C/C'),  # trans double bond
        Chem.MolFromSmarts('C/C=C\C'),  # cis double bond
    ]
    
    total_unsaturations = 0
    for pattern in unsaturation_patterns:
        if pattern is not None:  # ensure pattern was created successfully
            matches = mol.GetSubstructMatches(pattern)
            total_unsaturations += len(matches)
    
    if total_unsaturations == 0:
        return False, "No C-C unsaturation found"

    return True, f"Unsaturated fatty acid anion with {total_unsaturations} unsaturation(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:2580',
                          'name': 'unsaturated fatty acid anion',
                          'definition': 'Any fatty acid anion containing at '
                                        'least one C-C unsaturated bond; '
                                        'formed by deprotonation of the '
                                        'carboxylic acid moiety.',
                          'parents': ['CHEBI:28868']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 55,
    'num_false_positives': 100,
    'num_true_negatives': 12241,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.3548387096774194,
    'recall': 0.9016393442622951,
    'f1': 0.5092592592592593,
    'accuracy': 0.9914529914529915}


"""
Classifies: CHEBI:134416 3-oxo fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_3_oxo_fatty_acid(smiles: str):
    """
    Determines if a molecule is a 3-oxo fatty acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a 3-oxo fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the carbonyl carbon atom
    carbonyl_carbon_idx = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetFormalCharge() == 0 and atom.GetTotalNumHs() == 0:
            neighbors = [neighbor.GetSymbol() for neighbor in atom.GetNeighbors()]
            if 'O' in neighbors and 'O' not in neighbors[neighbors.index('O'):]:
                carbonyl_carbon_idx = atom.GetIdx()
                break

    if carbonyl_carbon_idx is None:
        return False, "No carbonyl carbon found"

    # Check if the carbonyl carbon is at position 3
    carbonyl_carbon = mol.GetAtomWithIdx(carbonyl_carbon_idx)
    chain = Chem.Mol.GetAtomBFs(mol, Chem.AtomType.AtomBFType.longCode)
    chain_len = max(len(chain[carbonyl_carbon_idx]), chain[carbonyl_carbon_idx].count('3')) + 1
    if chain_len != 4:
        return False, "Carbonyl carbon is not at position 3"

    # Check if the molecule has a carboxyl group
    carboxyl_found = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and atom.GetFormalCharge() == 0 and atom.GetTotalNumHs() == 0:
            neighbors = [neighbor.GetSymbol() for neighbor in atom.GetNeighbors()]
            if 'O' in neighbors and 'O' in neighbors[neighbors.index('O')+1:]:
                carboxyl_found = True
                break

    if not carboxyl_found:
        return False, "No carboxyl group found"

    return True, "The molecule is a 3-oxo fatty acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:134416',
                          'name': '3-oxo fatty acid',
                          'definition': 'Any oxo fatty acid in which an oxo '
                                        'substituent is located at position 3.',
                          'parents': ['CHEBI:47881', 'CHEBI:59644']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183924,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630012234}


"""
Classifies: CHEBI:24727 hydroxynaphthalene
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxynaphthalene(smiles: str):
    """
    Determines if a molecule is a hydroxynaphthalene (naphthalene with one or more OH groups).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a hydroxynaphthalene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for naphthalene core
    # Naphthalene consists of two fused benzene rings
    rings = mol.GetRingInfo()
    
    # Find aromatic 6-membered rings
    aromatic_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings.append(set(ring))

    # Need at least 2 aromatic rings
    if len(aromatic_rings) < 2:
        return False, "No naphthalene core found (requires 2 fused aromatic rings)"

    # Check if there are two fused 6-membered aromatic rings
    found_naphthalene = False
    for i in range(len(aromatic_rings)):
        for j in range(i+1, len(aromatic_rings)):
            # Two rings are fused if they share exactly 2 atoms
            shared_atoms = aromatic_rings[i].intersection(aromatic_rings[j])
            if len(shared_atoms) == 2:
                # Check if the fused rings form a naphthalene (all carbons)
                ring1_atoms = [mol.GetAtomWithIdx(idx) for idx in aromatic_rings[i]]
                ring2_atoms = [mol.GetAtomWithIdx(idx) for idx in aromatic_rings[j]]
                if all(atom.GetSymbol() == 'C' for atom in ring1_atoms + ring2_atoms):
                    found_naphthalene = True
                    naphthalene_atoms = aromatic_rings[i].union(aromatic_rings[j])
                    break
        if found_naphthalene:
            break

    if not found_naphthalene:
        return False, "No naphthalene core found"

    # Check for OH groups
    hydroxyl_count = 0
    hydroxyl_positions = []
    
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O':
            # Check if oxygen is part of an OH group
            if atom.GetTotalNumHs() > 0:  # Has at least one H
                # Check if OH is attached to naphthalene core
                for neighbor in atom.GetNeighbors():
                    if neighbor.GetIdx() in naphthalene_atoms:
                        hydroxyl_count += 1
                        hydroxyl_positions.append(str(neighbor.GetIdx()))

    if hydroxyl_count == 0:
        return False, "No hydroxyl groups found on naphthalene core"

    return True, f"Hydroxynaphthalene with {hydroxyl_count} OH group(s) at position(s): {', '.join(hydroxyl_positions)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24727',
                          'name': 'hydroxynaphthalene',
                          'definition': 'Any member of the class of  '
                                        'naphthalenes that is naphthalene '
                                        'carrying one or more hydroxy groups.',
                          'parents': ['CHEBI:25477', 'CHEBI:33853']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 24,
    'num_false_positives': 100,
    'num_true_negatives': 15866,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.1935483870967742,
    'recall': 1.0,
    'f1': 0.3243243243243243,
    'accuracy': 0.9937460913070669}


"""
Classifies: CHEBI:22512 aminoimidazole
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_aminoimidazole(smiles: str):
    """
    Determines if a molecule is an aminoimidazole (imidazole with at least one amino substituent).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an aminoimidazole, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for imidazole ring
    imidazole_pattern = Chem.MolFromSmarts('[nH]1cncc1')  # Basic imidazole pattern
    if not mol.HasSubstructMatch(imidazole_pattern):
        return False, "No imidazole ring found"

    # Check for amino substituent (-NH2)
    amino_pattern = Chem.MolFromSmarts('N[H2]')
    if not mol.HasSubstructMatch(amino_pattern):
        return False, "No amino substituent found"

    # Find imidazole rings
    imidazole_matches = mol.GetSubstructMatches(imidazole_pattern)
    
    # Find amino groups
    amino_matches = mol.GetSubstructMatches(amino_pattern)
    
    # Check if any amino group is attached to imidazole ring
    for imidazole_match in imidazole_matches:
        imidazole_atoms = set(imidazole_match)
        
        for amino_match in amino_matches:
            amino_n = amino_match[0]
            # Get neighbors of amino nitrogen
            amino_neighbors = [x.GetIdx() for x in mol.GetAtomWithIdx(amino_n).GetNeighbors()]
            
            # Check if amino group is connected to imidazole ring
            for neighbor in amino_neighbors:
                if neighbor in imidazole_atoms:
                    position = get_amino_position(mol, neighbor, imidazole_match)
                    return True, f"Aminoimidazole found with amino group at position {position}"
                
    return False, "Amino group not attached to imidazole ring"

def get_amino_position(mol, attachment_point, imidazole_atoms):
    """Helper function to determine position of amino substituent on imidazole ring"""
    # Convert tuple to list for easier indexing
    imidazole_list = list(imidazole_atoms)
    
    # Find the nitrogen atoms in the imidazole ring
    n_atoms = [i for i, idx in enumerate(imidazole_list) if mol.GetAtomWithIdx(idx).GetSymbol() == 'N']
    
    # The first nitrogen (NH) is considered position 1
    # Renumber atoms starting from NH
    nh_pos = n_atoms[0]  # Position of NH in the list
    reordered = imidazole_list[nh_pos:] + imidazole_list[:nh_pos]
    
    # Find position of attachment point in reordered list
    try:
        position = reordered.index(attachment_point) + 1
        return position
    except ValueError:
        return "unknown"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22512',
                          'name': 'aminoimidazole',
                          'definition': 'Any member of the class of  '
                                        'imidazoles carrying at least one '
                                        'amino substituent.',
                          'parents': ['CHEBI:24780']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 6,
    'num_true_negatives': 183908,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999565018812936}


"""
Classifies: CHEBI:143016 ultra-long-chain primary fatty alcohol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_ultra_long_chain_primary_fatty_alcohol(smiles: str):
    """
    Determines if a molecule is an ultra-long-chain primary fatty alcohol.
    Ultra-long-chain primary fatty alcohols are defined as any primary fatty alcohol with a chain length greater than C27.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is an ultra-long-chain primary fatty alcohol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains an OH group
    oh_atom = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetHybridization() == Chem.HybridizationType.SP3:
            oh_atom = atom
            break

    if oh_atom is None:
        return False, "Molecule does not contain an OH group"

    # Check if the OH group is terminal
    if len(oh_atom.GetNeighbors()) != 1:
        return False, "OH group is not terminal"

    # Check if the molecule is a primary alcohol
    carbon_atom = oh_atom.GetNeighbors()[0]
    if carbon_atom.GetDegree() != 2:
        return False, "Not a primary alcohol"

    # Calculate the chain length
    chain_length = len(Chem.MolToSmiles(mol).split('.')[0]) - 1  # Subtract 1 to account for the OH group

    if chain_length > 27:
        return True, f"Ultra-long-chain primary fatty alcohol with chain length {chain_length}"
    else:
        return False, f"Chain length is {chain_length}, which is not greater than 27"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:143016',
                          'name': 'ultra-long-chain primary fatty alcohol',
                          'definition': 'Any primary fatty alcohol with a '
                                        'chain length greater than C27.',
                          'parents': ['CHEBI:142622', 'CHEBI:197505']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: module 'rdkit.Chem.rdMolDescriptors' has no "
               "attribute 'GetSMILESLength'",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 100,
    'num_true_negatives': 995,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.009900990099009901,
    'recall': 1.0,
    'f1': 0.0196078431372549,
    'accuracy': 0.9087591240875912}